#ifndef Vector_h
#define Vector_h

template <class T> class Vector3 {
public:	
	T x, y, z;


	Vector3(){}

	Vector3( T i, T j, T k): x(i), y(j), z(k){
	}

	Vector3 cross(const Vector3 &other){
		Vector3 res;

		res.x=(y*other.z)-(z*other.y);
		res.y=(z*other.x)-(x*other.z);
		res.z=(x*other.y)-(y*other.x);

		return res;

	}
	float lengthSquared() const {
      return (x*x + y*y + z*z);
   }

   //: Return vector's length
   float length() const {
      return sqrtf(lengthSquared());
   }


	void normalize() {
      float len = length();
      float len_inv = 1.0f/len;
      x *= len_inv;
      y *= len_inv;
      z *= len_inv;
   }

	Vector3 &operator () (T i, T j, T k){
	  x=i; y=j; z=k;
	  return *this;
	}

	Vector3 operator + ( const Vector3 &other ){
		Vector3 res;

		res.x = x + other.x;
		res.y = y + other.y;
		res.z = z + other.z;

		return res;
	}

	Vector3 operator - ( const Vector3 &other ){
		Vector3 res;

		res.x = x - other.x;
		res.y = y - other.y;
		res.z = z - other.z;

		return res;
	}

	Vector3 operator * ( const Vector3 &other ) {
		Vector3 res;

		res.x = x * other.x;
		res.y = y * other.y;
		res.z = z * other.z;
		
		return res;
	}
	
	Vector3 operator / ( const Vector3 &other ) {
	  Vector3 res;
	  
	  res.x = x / other.x;
	  res.y = y / other.y;
	  res.z = z / other.z;
	  
	  return res;
	}
	
	Vector3 operator * ( const T scalar ) {
	  Vector3 res;
	  
	  res.x = x * scalar;
	  res.y = y * scalar;
	  res.z = z * scalar;
	  
	  return res;
	}
	
	Vector3 operator / ( const T scalar ) {
	  Vector3 res;

	  res.x = x / scalar;
	  res.y = y / scalar;
	  res.z = z / scalar;
	  
	  return res;
	}
	
	bool operator == ( const Vector3 &other ) {
	  if(other.x!=x)
	    return 0;
	  
	  if(other.y!=y)
	    return 0;
	  
	  if(other.z!=z)
	    return 0;
	  
	  return 1;
	}
	
	bool operator != ( const Vector3 &other ) {
	  if(other.x!=x)
	    return 1;
	  
	  if(other.y!=y)
	    return 1;

	  if(other.z!=z)
	    return 1;
	  
	  return 0;
	}

	Vector3 operator - ( ) {
	  Vector3 res;

	  res.x = -x;
	  res.y = -y;
	  res.z = -z;
	  
	  return res;
	}
};

typedef Vector3<float> FVector;
typedef Vector3<int> IVector;

static FVector operator /(const FVector fv, const IVector iv){
  FVector res;
  
  res.x = fv.x / iv.x;
  res.y = fv.y / iv.y;
  res.z = fv.z / iv.z;
  
  return res;
}


#endif
